import type {
	FilenamesForPackageFiles,
	FilenamesForReadmeFiles,
	DataForReadmeFiles,
	DataForPackageFiles,
	EnhancedPackagesData,
	EnhancedReadmesData,
} from './types.js'
interface Options {
	/** the directory that we wish to do our work on, defaults to `process.cwd()` */
	cwd?: string
	/** the log function to use, first argument being the log level */
	log?: Function
}
export declare class Projectz {
	/** our log function to use (logLevel, ...messages) */
	protected readonly log: Function
	/** the current working directory (the path) that projectz is working on */
	protected readonly cwd: string
	/**
	 * The absolute paths for all the package files.
	 * Should be arranged in the order of merging preference.
	 */
	protected readonly filenamesForPackageFiles: FilenamesForPackageFiles
	/** the data for each of our package files */
	protected readonly dataForPackageFiles: DataForPackageFiles
	/** the absolute paths for all the meta files */
	protected readonly filenamesForReadmeFiles: FilenamesForReadmeFiles
	/** the data for each of our readme files */
	protected readonly dataForReadmeFiles: DataForReadmeFiles
	constructor(opts: Options)
	/** Compile the project */
	compile(): Promise<void>
	/** Load in the paths we have specified */
	protected loadPaths(): Promise<void>
	/** Merge and enhance the packages data */
	protected enhancePackagesData(): Promise<EnhancedPackagesData>
	/** Merge and enhance the readmes data */
	protected enhanceReadmesData(
		data: EnhancedPackagesData
	): Promise<DataForReadmeFiles>
	/** Save the data we've loaded into the files */
	protected save(
		enhancedPackagesData: EnhancedPackagesData,
		enhancedReadmesData: EnhancedReadmesData
	): Promise<void>
}
export {}
//# sourceMappingURL=index.d.ts.map
