'use strict'
Object.defineProperty(exports, '__esModule', { value: true })
exports.getContributeSection = exports.getBackerFile = exports.getBackerSection = void 0
const util_js_1 = require('./util.js')
const badge_js_1 = require('./badge.js')
function getSponsorsText(data) {
	let result = ''
	if (data.sponsors.length === 0) {
		// ignore
		result +=
			'No sponsors yet! Will you be the first?\n\n' +
			badge_js_1.getBadgesInCategory('funding', data)
	} else {
		result +=
			'These amazing people have contributed finances to this project:\n\n' +
			util_js_1.getPeopleHTML(data.sponsors) +
			`\n\nBecome a sponsor!\n\n${badge_js_1.getBadgesInCategory(
				'funding',
				data
			)}`
	}
	return result
}
function getMaintainersText(data) {
	let result = ''
	if (data.maintainers.length === 0) {
		// ignore
		result += 'No maintainers yet! Will you be the first?'
	} else {
		result +=
			'These amazing people are maintaining this project:\n\n' +
			util_js_1.getPeopleHTML(data.maintainers, {
				displayContributions: true,
				githubRepoSlug: data.github.slug,
			})
	}
	return result
}
function getContributeLink(data, optional = false) {
	// Prepare
	const file = data.filenamesForReadmeFiles.contributing
	if (!file) {
		if (optional) {
			return ''
		} else {
			throw new Error(
				'Contributing section requires a CONTRIBUTING file to exist'
			)
		}
	}
	const url = util_js_1.getFileUrl(data, file)
	const text = `Discover how you can contribute by heading on over to the <code>${file}</code> file.`
	// Return
	return util_js_1.getLink({ url, text })
}
function getContributorsText(data) {
	let result = ''
	if (data.contributors.length === 0) {
		// ignore
		result +=
			'No contributors yet! Will you be the first?' +
			`\n\n${getContributeLink(data, true)}`
	} else {
		result +=
			'These amazing people have contributed code to this project:\n\n' +
			util_js_1.getPeopleHTML(data.contributors, {
				displayContributions: true,
				githubRepoSlug: data.github.slug,
			}) +
			`\n\n${getContributeLink(data, true)}`
	}
	return result
}
function getBackerSection(data) {
	// Prepare
	const result = [
		'<h2>Backers</h2>',
		'',
		'<h3>Maintainers</h3>',
		'',
		getMaintainersText(data),
		'',
		'<h3>Sponsors</h3>',
		'',
		getSponsorsText(data),
		'',
		'<h3>Contributors</h3>',
		'',
		getContributorsText(data),
	].join('\n')
	// Return
	return result
}
exports.getBackerSection = getBackerSection
function getBackerFile(data) {
	// Prepare
	const result = [
		'<h1>Backers</h1>',
		'',
		'<h2>Maintainers</h2>',
		'',
		getMaintainersText(data),
		'',
		'<h2>Sponsors</h2>',
		'',
		getSponsorsText(data),
		'',
		'<h2>Contributors</h2>',
		'',
		getContributorsText(data),
	].join('\n')
	// Return
	return result
}
exports.getBackerFile = getBackerFile
function getContributeSection(data) {
	// Prepare
	const result = ['<h2>Contribute</h2>', '', getContributeLink(data)].join('\n')
	// Return
	return result
}
exports.getContributeSection = getContributeSection
