'use strict'
var __importDefault =
	(this && this.__importDefault) ||
	function (mod) {
		return mod && mod.__esModule ? mod : { default: mod }
	}
Object.defineProperty(exports, '__esModule', { value: true })
// Import process for deno compat
const process_1 = __importDefault(require('process'))
// Import caterpillar [Caterpillar](https://github.com/bevry/caterpillar) for logging
const caterpillar_1 = require('caterpillar')
// Import the package used to get the value of CLI arguments
const get_cli_arg_1 = __importDefault(require('get-cli-arg'))
// Import out projectz utility
const index_js_1 = require('./index.js')
async function main() {
	// Compile
	if (process_1.default.argv.includes('compile')) {
		// fetch
		const p = get_cli_arg_1.default('path')
		const d = get_cli_arg_1.default('verbose')
		// bc upgrade
		if (process_1.default.argv.includes('-p')) {
			console.log(
				'projecz now requires -p argument to be specifie via --path=value'
			)
			return process_1.default.exit(1)
		}
		if (process_1.default.argv.includes('-d')) {
			console.log(
				'projecz now requires -d argument to be specifie via --verbose'
			)
			return process_1.default.exit(1)
		}
		// Prepare our logging configuration
		const level = d ? 7 : 6
		// Setup our logging
		const logger = new caterpillar_1.Logger({ lineLevel: level })
		const filter = new caterpillar_1.Filter({ filterLevel: level })
		const human = new caterpillar_1.Human()
		// Pipe logger output to filter, then filter output to stdout
		logger.pipe(filter).pipe(human).pipe(process_1.default.stdout)
		// Compile
		try {
			const project = new index_js_1.Projectz({
				log: logger.log.bind(logger),
				cwd: p || null,
			})
			await project.compile()
		} catch (err) {
			// error
			logger.log('err', err.stack)
			return process_1.default.exit(1)
		}
		// Done
		logger.log('info', 'Completed successfully')
	} else {
		// output help
		console.log(
			'projectz compile: merge our data files and compile our meta files'
		)
		console.log('\t--verbose\tOutputs verbose logging')
		console.log(
			'\t--path=value\tPath to the project that you wish to work with, defaults to the current working directory'
		)
		return process_1.default.exit(1)
	}
}
main()
