'use strict'
/* :: declare type Person = Object; */
/* :: declare type PersonOptions = {displayCopyright?:boolean; displayYears?:boolean; githubSlug?:string}; */
Object.defineProperty(exports, '__esModule', { value: true })
exports.trim = exports.replaceSection = exports.getLink = exports.getFileUrl = exports.getPeopleTextArray = exports.getPeopleHTML = exports.getGithubSlug = void 0
function getGithubSlug(data) {
	let match = null
	if (typeof data.repository === 'string') {
		match = data.repository.match(/^(?:github:)?([^/:]+\/[^/:]+)$/)
	} else {
		let url = null
		if (data.repository && typeof data.repository.url === 'string') {
			url = data.repository && data.repository.url
		} else if (typeof data.homepage === 'string') {
			url = data.homepage
		} else {
			return null
		}
		match = url.match(/github\.com[/:]([^/:]+\/[^/:]+?)(?:\.git|\/)?$/)
	}
	return (match && match[1]) || null
}
exports.getGithubSlug = getGithubSlug
function getPeopleHTML(people, opts) {
	if (people.length === 0) {
		return ''
	} else {
		return (
			'<ul>' +
			people
				.map(function (person) {
					return '<li>' + person.toHTML(opts) + '</li>'
				})
				.join('\n') +
			'</ul>'
		)
	}
}
exports.getPeopleHTML = getPeopleHTML
function getPeopleTextArray(people, opts) {
	if (people.length === 0) {
		return []
	} else {
		const textArray = []
		people.forEach(function (person) {
			if (!person.name || person.name === 'null') {
				throw new Error(
					`For some reason the person doesn't have a name: ${JSON.stringify(
						person,
						null,
						'  '
					)}`
				)
			}
			const text = person.toString(opts)
			if (text) textArray.push(text)
		})
		return textArray
	}
}
exports.getPeopleTextArray = getPeopleTextArray
function getFileUrl(data, filename) {
	if (data.github.slug) {
		return `https://github.com/${data.github.slug}/blob/master/${filename}#files`
	} else {
		throw new Error(
			'File links are currently only supported for github repositories'
		)
	}
}
exports.getFileUrl = getFileUrl
function getLink({ url, text, title }) {
	if (!url || !text) {
		throw new Error('Links must have both a url and text')
	}
	if (title) {
		return `<a href="${url}" title="${title}">${text}</a>`
	} else {
		return `<a href="${url}">${text}</a>`
	}
}
exports.getLink = getLink
// @todo replace this with bevry/ropo
function replaceSection(names, source, inject) {
	let regexName, sectionName
	if (Array.isArray(names)) {
		regexName = '(' + names.join('|') + ')'
		sectionName = names[0]
	} else {
		regexName = sectionName = names
	}
	sectionName = sectionName.toUpperCase()
	/* eslint indent:0 */
	const regex = new RegExp(
		[
			'^(',
			`<!--\\s*${regexName}\\s*-->`,
			'|',
			`<!--\\s*${regexName}/\\s*-->`,
			'[\\s\\S]*?',
			`<!--\\s*/${regexName}\\s*-->`,
			')\\s+',
		].join(''),
		'gim'
	)
	function replace() {
		const result = typeof inject === 'function' ? inject() : inject
		return `<!-- ${sectionName}/ -->\n\n${result}\n\n<!-- /${sectionName} -->\n\n\n`
	}
	const result = source.replace(regex, replace)
	return result
}
exports.replaceSection = replaceSection
function trim(str) {
	return str.replace(/^\s+|\s+$/g, '')
}
exports.trim = trim
